/*!
 * vue-router-layout v0.1.3
 * https://github.com/ktsn/vue-router-layout
 *
 * @license
 * Copyright (c) 2018 katashin
 * Released under the MIT license
 * https://github.com/ktsn/vue-router-layout/blob/master/LICENSE
 */
(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('vue')) :
    typeof define === 'function' && define.amd ? define(['exports', 'vue'], factory) :
    (factory((global.VueRouterLayout = {}),global.Vue));
}(this, (function (exports,Vue) { 'use strict';

    Vue = Vue && Vue.hasOwnProperty('default') ? Vue['default'] : Vue;

    /**
     * Find which layout the component should render.
     * If the component is not specified layout name, `default` is used.
     * Otherwise return undefined.
     */
    function resolveLayoutName(matched) {
        var defaultName = 'default';
        var last = matched[matched.length - 1];
        if (!last) {
            return;
        }
        var Component = last.components.default;
        if (!Component) {
            return;
        }
        var isAsync = typeof Component === 'function' && !Component.options;
        if (isAsync) {
            return;
        }
        var isCtor = typeof Component === 'function' && Component.options;
        var layoutName = isCtor ? Component.options.layout : Component.layout;
        return layoutName || defaultName;
    }
    function createRouterLayout(resolve) {
        return Vue.extend({
            name: 'RouterLayout',
            data: function () {
                return {
                    layoutName: undefined,
                    layouts: Object.create(null)
                };
            },
            watch: {
                layoutName: function (name) {
                    if (!this.layouts[name]) {
                        this.$set(this.layouts, name, function () { return resolve(name); });
                    }
                }
            },
            provide: function () {
                var _this = this;
                return {
                    $_routerLayout_notifyUpdate: function () {
                        var matched = _this.$route.matched;
                        _this.layoutName = resolveLayoutName(matched) || _this.layoutName;
                    }
                };
            },
            beforeRouteEnter: function (to, _from, next) {
                var _this = this;
                next(function (vm) {
                    vm.layoutName = resolveLayoutName(to.matched) || _this.layoutName;
                });
            },
            beforeRouteUpdate: function (to, _from, next) {
                this.layoutName = resolveLayoutName(to.matched) || this.layoutName;
                next();
            },
            render: function (h) {
                var layout = this.layoutName && this.layouts[this.layoutName];
                if (!layout) {
                    return h();
                }
                return h(layout, {
                    key: this.layoutName
                });
            }
        });
    }
    Vue.mixin({
        inject: {
            $_routerLayout_notifyUpdate: {
                default: null
            }
        },
        beforeUpdate: function () {
            var notify = this.$_routerLayout_notifyUpdate;
            if (notify) {
                notify();
            }
        }
    });

    exports.createRouterLayout = createRouterLayout;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
